/* TroiaPreview
 * version 1.0 date 2005-03-24
 * Copyright (C) 2003-2005 Stefan Schuermans <1stein@schuermans.info>
 * Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
 */

#include <string.h>
#include <BlinkenFrame.h>
#include <BlinkenMovie.h>

#include "Tools.h"
#include "Pixel.h"
#include "Frame.h"
#include "Movie.h"

//type for a movie
typedef struct _tMovie
{
  int Active; //if movie is active
  stBlinkenMovie * pMovie; //the movie, may be NULL
  int Width, Height; //dimensions of movie
  int FrameCnt; //numer of frames of the movie
  int FrameNo; //current frame of the movie
  int FrameTime; //time (in ms) this frame is being shown
  double LastTime; //last time this movie was processed
  tPixel3D TopLeft; //starting point of movie
  unsigned char Dir; //direction of movie
} tMovie;

//list of movies
tMovie Movies[MovieCnt];

//initialize movies
void MovieInit( )
{
  int I;

  //disable all movies
  for( I = 0; I < MovieCnt; I++ )
  {
    Movies[I].Active = 0; //not active
    Movies[I].pMovie = NULL; //no movie
    Movies[I].Width = 0;
    Movies[I].Height = 0;
    Movies[I].FrameCnt = 0;
    Movies[I].FrameNo = 0;
    Movies[I].FrameTime = 0;
    Movies[I].LastTime = 0.0;
    Movies[I].TopLeft.Wall = 0; //invalid pixel
    Movies[I].Dir = 0; //invalid direction
  }
}

//cleanup movies
void MovieExit( )
{
  int I;

  //free all movies
  for( I = 0; I < MovieCnt; I++ )
  {
    if( Movies[I].pMovie != NULL )
    {
      BlinkenMovieFree( Movies[I].pMovie );
      Movies[I].pMovie = NULL;
    }
    Movies[I].Width = 0;
    Movies[I].Height = 0;
    Movies[I].FrameCnt = 0;
    Movies[I].FrameNo = 0;
    Movies[I].FrameTime = 0;
  }
}

//check if movie is active
int MovieGetActive( unsigned int MovieNo )
{
  //check movie number
  if( MovieNo >= MovieCnt )
    return 0;
  //return if movie active
  return Movies[MovieNo].Active;
}

//enable / disable movie
void MovieSetActive( unsigned int MovieNo, int Active )
{
  //check movie number
  if( MovieNo >= MovieCnt )
    return;
  //save if movie is active now
  Movies[MovieNo].Active = Active;
  //rewind movie
  Movies[MovieNo].FrameNo = 0;
  Movies[MovieNo].FrameTime = 0;
  Movies[MovieNo].LastTime = TimeGet( );
}

//load movie file
//returns 0 on success, -1 on error
int MovieLoadFile( unsigned int MovieNo, char * pFilename )
{
  //check movie number
  if( MovieNo >= MovieCnt )
    return -1;
  //free old movie
  if( Movies[MovieNo].pMovie != NULL )
    BlinkenMovieFree( Movies[MovieNo].pMovie );
  Movies[MovieNo].Width = 0;
  Movies[MovieNo].Height = 0;
  Movies[MovieNo].FrameCnt = 0;
  Movies[MovieNo].FrameNo = 0;
  Movies[MovieNo].FrameTime = 0;
  //load new movie
  Movies[MovieNo].pMovie = BlinkenMovieLoad( pFilename );
  if( Movies[MovieNo].pMovie == NULL )
    return -1;
  //resize movie to one channel and to 2
  Movies[MovieNo].Width = BlinkenMovieGetWidth( Movies[MovieNo].pMovie );
  Movies[MovieNo].Height = BlinkenMovieGetHeight( Movies[MovieNo].pMovie ); 
  BlinkenMovieResize( Movies[MovieNo].pMovie, Movies[MovieNo].Height, Movies[MovieNo].Width, 1, 255 );
  //get numer of frames and dimensions
  Movies[MovieNo].FrameCnt = BlinkenMovieGetFrameCnt( Movies[MovieNo].pMovie );
  //rewind movie
  Movies[MovieNo].FrameNo = 0;
  Movies[MovieNo].FrameTime = 0;
  Movies[MovieNo].LastTime = TimeGet( );
  return 0;
}

//get movie position and direction
void MovieGetPos( unsigned int MovieNo, tPixel3D * pPix, unsigned char * pDir )
{
  //check movie number
  if( MovieNo >= MovieCnt )
    return;
  //return position and direction
  *pPix = Movies[MovieNo].TopLeft;
  *pDir = Movies[MovieNo].Dir;
}

//set movie position and direction
void MovieSetPos( unsigned int MovieNo, tPixel3D Pix, unsigned char Dir )
{
  //check movie number
  if( MovieNo >= MovieCnt )
    return;
  //save new position and direction
  Movies[MovieNo].TopLeft = Pix;
  Movies[MovieNo].Dir = Dir;
}

//draw a movie
static void MovieDraw( tMovie * pMovie, double CurrentTime )
{
  int Time, Duration, Y, X;
  stBlinkenFrame * pFrame;
  tPixel3D Column, Pixel;
  unsigned char Right, Down;

  //do nothing if movie inactive or has no frames
  if( ! pMovie->Active || pMovie->FrameCnt <= 0 )
    return;

  //get time (in ms) since last processing
  Time = (int)((CurrentTime - pMovie->LastTime) * 1000);
  if( Time < 0 )
    Time = 0;
  pMovie->LastTime = CurrentTime;
  //add this time to frame time
  pMovie->FrameTime += Time;

  //find frame to display
  for( ; ; )
  {
    //get current frame and its duration
    if( pMovie->FrameNo >= pMovie->FrameCnt ) //loop movie
      pMovie->FrameNo = 0;
    pFrame = BlinkenMovieGetFrame( pMovie->pMovie, pMovie->FrameNo );
    if( pFrame == NULL )
      return;
    Duration = BlinkenFrameGetDuration( pFrame );
    //frame has still to be shown
    if( pMovie->FrameTime < Duration )
      break;
    //next frame
    pMovie->FrameTime -= Duration;
    pMovie->FrameNo++;
  }

  //draw current frame to temporary frame buffer

  //start rightwards at top left
  Column = pMovie->TopLeft;
  Right = pMovie->Dir;
  for( X = 0; X < pMovie->Width; X++ )
  {
    //start downwards at top of column
    Pixel = Column;
    Down = DirRotate( Right, 2 );
    for( Y = 0; Y < pMovie->Height; Y++ )
    {
      //draw pixel
      FrameTmpSetPixel( Pixel, BlinkenFrameGetPixel( pFrame, Y, X, 0 ) );
      //go one pixel down
      Pixel3DMove( Pixel, Down, &Pixel, &Down );
    }
    //go one pixel right
    Pixel3DMove( Column, Right, &Column, &Right );
  }
}

//called periodically to do some work
void MovieTask( )
{
  double CurrentTime;
  int I;

  //get time
  CurrentTime = TimeGet( );

  //draw all active movies
  for( I = 0; I < MovieCnt; I++ )
    MovieDraw( &Movies[I], CurrentTime );
}
